@@ -139,6 +139,8 @@ gpp_clip (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 		"wind", rule,
 		NULL);
 
+	gp_gc_set_data (priv->gc, clip);
+
 	gp_path_unref (path);
 
 	return 1;
@@ -272,6 +274,30 @@ gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * gl
 	return 0;
 }
 
+static int
+gpp_gsave (GnomePrintContext *ctx)
+{
+	GnomePrintPreview *pp;
+
+	pp = GNOME_PRINT_PREVIEW (ctx);
+
+	gp_gc_gsave (pp->priv->gc);
+
+	return GNOME_PRINT_OK;
+}
+
+static int
+gpp_grestore (GnomePrintContext *ctx)
+{
+	GnomePrintPreview *pp;
+
+	pp = GNOME_PRINT_PREVIEW (ctx);
+
+	gp_gc_grestore (pp->priv->gc);
+
+	return GNOME_PRINT_OK;
+}
+
 static int
 gpp_close (GnomePrintContext *pc)
 {
@@ -321,6 +347,9 @@ class_init (GnomePrintPreviewClass *klass)
 
 	pc_class->glyphlist = gpp_glyphlist;
 	
+	pc_class->gsave = gpp_gsave;
+	pc_class->grestore = gpp_grestore;
+
 	pc_class->close = gpp_close;
 }
 
@@ -363,7 +392,7 @@ GnomePrintContext *gnome_print_preview_new_full (GnomePrintConfig *config, Gnome
 	g_return_val_if_fail (region != NULL, NULL);
 
 	ctx = g_object_new (GNOME_TYPE_PRINT_PREVIEW, NULL);
-	ret = gnome_print_context_construct (ctx, GPA_NODE (config));
+	ret = gnome_print_context_construct (ctx, config);
 	if (ret != GNOME_PRINT_OK) {
 		g_object_unref (ctx);
 		return NULL;
@@ -413,6 +442,7 @@ gnome_print_preview_new (GnomePrintConfig *config, GnomeCanvas *canvas)
 {
 	ArtDRect bbox;
 	gdouble page2root[6];
+	const GnomePrintUnit *unit;
 	
 	g_return_val_if_fail (config != NULL, NULL);
 	g_return_val_if_fail (canvas != NULL, NULL);
@@ -426,8 +456,12 @@ gnome_print_preview_new (GnomePrintConfig *config, GnomeCanvas *canvas)
 		bbox.y0 = 0.0;
 		bbox.x1 = 21.0 * (72.0 / 2.54);
 		bbox.y1 = 29.7 * (72.0 / 2.54);
-		gnome_print_config_get_double (config, GNOME_PRINT_KEY_PAPER_WIDTH, &bbox.x1);
-		gnome_print_config_get_double (config, GNOME_PRINT_KEY_PAPER_HEIGHT, &bbox.y1);
+		if (gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_WIDTH, &bbox.x1, &unit)) {
+			gnome_print_convert_distance (&bbox.x1, unit, GNOME_PRINT_PS_UNIT);
+		}
+		if (gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_HEIGHT, &bbox.y1, &unit)) {
+			gnome_print_convert_distance (&bbox.y1, unit, GNOME_PRINT_PS_UNIT);
+		}
 	}
 
 	art_affine_scale (page2root, 1.0, -1.0);
